/******************************************************************************
 * Copyright 2022 TypeFox GmbH
 * This program and the accompanying materials are made available under the
 * terms of the MIT License, which is available in the project root.
 ******************************************************************************/
import { startLanguageServer, EmptyFileSystem, DocumentState } from 'langium';
import { BrowserMessageReader, BrowserMessageWriter, NotificationType, createConnection } from 'vscode-languageserver/browser.js';
import { createMiniLogoServices } from './minilogo-module.js';
import { getCommands } from './minilogo-actions.js';
import { generateMiniLogoCmds } from '../generator/generator.js';
/* browser specific setup code */
const messageReader = new BrowserMessageReader(self);
const messageWriter = new BrowserMessageWriter(self);
const connection = createConnection(messageReader, messageWriter);
// Inject the shared services and language-specific services
const { shared, MiniLogo } = createMiniLogoServices({ connection, ...EmptyFileSystem });
// Start the language server with the shared services
startLanguageServer(shared);
const documentChangeNotification = new NotificationType('browser/DocumentChange');
const jsonSerializer = MiniLogo.serializer.JsonSerializer;
shared.workspace.DocumentBuilder.onBuildPhase(DocumentState.Validated, documents => {
    var _a;
    for (const document of documents) {
        const module = document.parseResult.value;
        let json = [];
        if (document.diagnostics === undefined || document.diagnostics.filter((i) => i.severity === 1).length === 0) {
            json = getCommands(generateMiniLogoCmds(module));
        }
        module.$commands = json;
        connection.sendNotification(documentChangeNotification, {
            uri: document.uri.toString(),
            content: jsonSerializer.serialize(module, { sourceText: true, textRegions: true }),
            diagnostics: (_a = document.diagnostics) !== null && _a !== void 0 ? _a : []
        });
    }
});
//# sourceMappingURL=main-browser.js.map