import { AbstractExecuteCommandHandler, createDefaultModule, createDefaultSharedModule, inject } from 'langium';
import { parseAndGenerate } from '../web/index.js';
import { MiniLogoGeneratedModule, MiniLogoGeneratedSharedModule } from './generated/module.js';
import { MiniLogoValidationRegistry, MiniLogoValidator } from './minilogo-validator.js';
/**
 * Dependency injection module that overrides Langium default services and contributes the
 * declared custom services. The Langium defaults can be partially specified to override only
 * selected services, while the custom services must be fully specified.
 */
export const MiniLogoModule = {
    validation: {
        ValidationRegistry: (services) => new MiniLogoValidationRegistry(services),
        MiniLogoValidator: () => new MiniLogoValidator()
    },
};
class MiniLogoCommandHandler extends AbstractExecuteCommandHandler {
    registerCommands(acceptor) {
        acceptor('parseAndGenerate', args => {
            // invoke generator on this data, and return response
            return parseAndGenerate(args[0]);
        });
    }
}
/**
 * Create the full set of services required by Langium.
 *
 * First inject the shared services by merging two modules:
 *  - Langium default shared services
 *  - Services generated by langium-cli
 *
 * Then inject the language-specific services by merging three modules:
 *  - Langium default language-specific services
 *  - Services generated by langium-cli
 *  - Services specified in this file
 *
 * @param context Optional module context with the LSP connection
 * @returns An object wrapping the shared services and the language-specific services
 */
export function createMiniLogoServices(context) {
    const shared = inject(createDefaultSharedModule(context), MiniLogoGeneratedSharedModule);
    const MiniLogo = inject(createDefaultModule({ shared }), MiniLogoGeneratedModule, MiniLogoModule);
    shared.lsp.ExecuteCommandHandler = new MiniLogoCommandHandler();
    shared.ServiceRegistry.register(MiniLogo);
    return { shared, MiniLogo };
}
//# sourceMappingURL=minilogo-module.js.map