import { ValidationRegistry } from 'langium';
import { isDef, isModel } from './generated/ast.js';
/**
 * Registry for validation checks.
 */
export class MiniLogoValidationRegistry extends ValidationRegistry {
    constructor(services) {
        super(services);
        const validator = services.validation.MiniLogoValidator;
        const checks = {
            Model: validator.checkUniqueDefs,
            Def: validator.checkUniqueParams
        };
        this.register(checks, validator);
    }
}
export class MiniLogoValidator {
    checkUniqueDefs(model, accept) {
        if (!isModel(model)) {
            throw new Error('Retrieve a non-model in validation');
        }
        const reported = new Set();
        model.defs.forEach(d => {
            if (reported.has(d.name)) {
                accept('error', `Def has non-unique name '${d.name}'.`, { node: d, property: 'name' });
            }
            reported.add(d.name);
        });
    }
    checkUniqueParams(def, accept) {
        if (!isDef(def)) {
            throw new Error('Retrieve a non-def in validation');
        }
        const reported = new Set();
        def.params.forEach(p => {
            if (reported.has(p.name)) {
                accept('error', `Param ${p.name} is non-unique for Def '${def.name}'`, { node: p, property: 'name' });
            }
            reported.add(p.name);
        });
    }
}
//# sourceMappingURL=minilogo-validator.js.map