import { createMiniLogoServices } from '../language-server/minilogo-module.js';
import { generateMiniLogoCmds } from '../generator/generator.js';
import { EmptyFileSystem } from 'langium';
import { URI } from 'langium';
/**
 * Parses a MiniLogo program & generates output as a list of Objects
 * @param miniLogoProgram MiniLogo program to parse
 * @returns Generated output from this MiniLogo program
 */
export async function parseAndGenerate(miniLogoProgram) {
    const services = createMiniLogoServices(EmptyFileSystem).MiniLogo;
    const model = await extractAstNodeFromString(miniLogoProgram, services);
    // generate mini logo drawing commands from the model
    const cmds = generateMiniLogoCmds(model);
    return Promise.resolve(cmds);
}
/**
 * Extracts an AST node from a virtual document, represented as a string
 * @param content Content to create virtual document from
 * @param services For constructing & building a virtual document
 * @returns A promise for the parsed result of the document
 */
async function extractAstNodeFromString(content, services) {
    var _a;
    const doc = services.shared.workspace.LangiumDocumentFactory.fromString(content, URI.parse('memory://minilogo.document'));
    await services.shared.workspace.DocumentBuilder.build([doc], { validation: true });
    return (_a = doc.parseResult) === null || _a === void 0 ? void 0 : _a.value;
}
//# sourceMappingURL=index.js.map